// Device.java
import java.io.*;

class FileIn implements In {
  private Reader reader;

  public FileIn(String filename) {
    try {
      reader = new FileReader(filename);
    } catch (FileNotFoundException e) {
      throw new RuntimeException(e);
    }
  }
  
  public int getc() {
    try {
      return reader.read();
    } catch (IOException e) {
      throw new RuntimeException(e);
    }
  }
}

class FileOut implements Out {
  private Writer writer;

  public FileOut(String filename) {
    try {
      writer = new FileWriter(filename);
    } catch (IOException e) {
      throw new RuntimeException(e);
    }
  }
  
  public void putc(char c) {
    try {
      writer.write((int)c);
    } catch (IOException e) {
      throw new RuntimeException(e);
    }
  }

  public void end() {
    try {
      writer.flush();
    } catch (IOException e) {
      throw new RuntimeException(e);
    }
  }
}

class DevNull implements Out {
  public void putc(char c) {
    ; // do nothing
  }

  public void end() {
    ; // do nothing
  }
}

class StdOut implements Out {
  public synchronized void putc(char c) {
    System.out.print(c);
  }

  public synchronized void end() {
    System.out.flush();
  }
}
