// Filter.java
abstract class Filter implements Runnable {
  protected In in;
  protected Out out;

  public Filter(In in, Out out) {
    this.in = in;
    this.out = out;
  }

  public abstract void run(); // runnable

  public void start() {
    (new Thread(this)).start();
  }
}

class Copy extends Filter {
  public Copy(In in, Out out) {
    super(in, out);
  }

  public void run() {
    int c;
    while ((c = in.getc()) != -1) {
      out.putc((char)c);
    }
    out.end();
  }
}

class Tolower extends Filter {
  public Tolower(In in, Out out) {
    super(in, out);
  }

  public void run() {
    int c;
    while ((c = in.getc()) != -1) {
      out.putc(Character.toLowerCase((char)c));
    }
    out.end();
  }
}

class Toupper extends Filter {
  public Toupper(In in, Out out) {
    super(in, out);
  }

  public void run() {
    int c;
    while ((c = in.getc()) != -1) {
      out.putc(Character.toUpperCase((char)c));
    }
    out.end();
  }
}

class Word extends Filter {
  public Word(In in, Out out) {
    super(in, out);
  }

  public void run() {
    boolean inWord = false;
    int c;
    while ((c = in.getc()) != -1) {
      if (Character.isLetterOrDigit((char)c)) {
	inWord = true;
	out.putc((char)c);
      } else {
	if (inWord) {
	  out.putc('\n');
	}
	inWord = false;
      }
    }
    out.end();
  }
}
